@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2022
-;; Version: 1.4
+;; Version: 1.5
 ;; Package-Requires: ((emacs "28.1") (compat "30"))
 ;; URL: https://github.com/minad/tempel
 ;; Keywords: abbrev, languages, tools, text
@@ -159,33 +159,46 @@ may be named with `tempel--name' or carry an evaluatable Lisp expression
   "M-<up>" #'tempel-previous
   "M-<down>" #'tempel-next)
 
-(defun tempel--print-template (elts)
-  "Print template ELTS."
+(defun tempel--print-template (template)
+  "Print TEMPLATE."
   (cl-loop
-   for elt in elts until (keywordp elt) concat
+   for elt in template until (keywordp elt) concat
    (pcase elt
      ('nil nil)
-     ((pred stringp) (propertize elt 'face 'completions-annotations))
-     (`(s ,name) (propertize (symbol-name name) 'face 'completions-annotations))
+     ((pred stringp) elt)
+     (`(s ,name) (symbol-name name))
      (`(,(or 'p 'P) ,_ ,name . ,noinsert)
-      (and (not (car noinsert))
-           (propertize (symbol-name name) 'face 'completions-annotations)))
-     ('> #(" " 0 1 (face completions-annotations)))
-     ('n> #("\n " 0 2 (face completions-annotations)))
-     ((or 'n '& '% 'o) #("\n" 0 1 (face completions-annotations)))
+      (and (not (car noinsert)) (symbol-name name)))
+     ('> " ")
+     ('n> "\n ")
+     ((or 'n '& '% 'o) "\n")
      (_ #("_" 0 1 (face shadow))))))
 
+(defun tempel--template-plist (template)
+  "Get property list from TEMPLATE list."
+  (cl-loop for x on template if (keywordp (car x)) return x))
+
+(defun tempel--insert-doc (template)
+  "Insert documentation of TEMPLATE."
+  (when-let ((doc (plist-get (tempel--template-plist template) :doc)))
+    (unless (eq ?\n (char-before)) (insert "\n"))
+    (insert (propertize "\n" 'face '(:overline t :height 0.1 :extend t)))
+    (insert doc)))
+
 (defun tempel--annotate (templates width sep name)
   "Annotate template NAME given the list of TEMPLATES.
 WIDTH and SEP configure the formatting."
   (when-let ((name (intern-soft name))
              (elts (cdr (assoc name templates))))
-    (concat sep (string-trim
-                 (truncate-string-to-width
-                  (replace-regexp-in-string
-                   "[ \t\n\r]+" #(" " 0 1 (face completions-annotations))
-                   (tempel--print-template elts))
-                  width)))))
+    (let ((ann (truncate-string-to-width
+                (string-trim
+                 (replace-regexp-in-string
+                  "[ \t\n\r]+" " "
+                  (or (plist-get (tempel--template-plist elts) :ann)
+                      (tempel--print-template elts))))
+                width)))
+      (add-face-text-property 0 (length ann) 'completions-annotations t ann)
+      (concat sep ann))))
 
 (defun tempel--info-buffer (templates fun name)
   "Create info buffer for template NAME.
@@ -401,9 +414,7 @@ If a field was added, return it."
 
 (defun tempel--insert (template region)
   "Insert TEMPLATE given the current REGION."
-  (let ((plist template))
-    (while (and plist (not (keywordp (car plist))))
-      (pop plist))
+  (let ((plist (tempel--template-plist template)))
     (eval (plist-get plist :pre) 'lexical)
     ;; TODO do we want to have the ability to reactivate snippets?
     (unless (eq buffer-undo-list t)
@@ -420,8 +431,8 @@ If a field was added, return it."
       (let ((st (cons nil nil))
             (ov (point))
             (tempel--inhibit-hooks t))
-        (while (and template (not (keywordp (car template))))
-          (tempel--element st region (pop template)))
+        (cl-loop for x in template until (keywordp x)
+                 do (tempel--element st region x))
         (setq ov (make-overlay ov (point) nil t))
         (push ov (car st))
         (overlay-put ov 'modification-hooks (list #'tempel--range-modified))
@@ -465,18 +476,25 @@ If a field was added, return it."
     (goto-char (point-max))
     (insert "\n)")
     (goto-char (point-min))
-    (let ((data (read (current-buffer))) result)
-      (while data
-        (let (modes plist templates)
-          (while (and (car data) (symbolp (car data)) (not (keywordp (car data))))
-            (push (pop data) modes))
-          (while (keywordp (car data))
-            (push (pop data) plist)
-            (push (pop data) plist))
-          (while (consp (car data))
-            (push (pop data) templates))
-          (push `(,(nreverse modes) ,(nreverse plist) . ,(nreverse templates)) result)))
-      result)))
+    (tempel--file-prepare (read (current-buffer)))))
+
+(defun tempel--file-prepare (data)
+  "Reorganize the template DATA from file.
+DATA must be a list (modes plist templates modes plist templates...).
+See the README, which provides an example for the file format of the
+template.eld file.  The return value is a list of (modes plist . templates)."
+  (let (result)
+    (while data
+      (let (modes plist templates)
+        (while (and (car data) (symbolp (car data)) (not (keywordp (car data))))
+          (push (pop data) modes))
+        (while (keywordp (car data))
+          (push (pop data) plist)
+          (push (pop data) plist))
+        (while (consp (car data))
+          (push (pop data) templates))
+        (push `(,(nreverse modes) ,(nreverse plist) . ,(nreverse templates)) result)))
+    result))
 
 (defun tempel-path-templates ()
   "Return templates defined in `tempel-path'.
@@ -496,10 +514,14 @@ This is meant to be a source in `tempel-template-sources'."
       (unless (equal (car tempel--path-templates) timestamps)
         (setq tempel--path-templates (cons timestamps
                                            (mapcan #'tempel--file-read files))))))
-  (cl-loop
-   for (modes plist . templates) in (cdr tempel--path-templates)
-   if (tempel--condition-p modes plist)
-   append templates))
+  (tempel--filter-templates (cdr tempel--path-templates)))
+
+(defun tempel--filter-templates (templates)
+  "Filter templates from TEMPLATES relevant to the current buffer.
+TEMPLATES must be a list in the form (modes plist . templates)."
+  (cl-loop for (modes plist . mode-templates) in templates
+           if (tempel--condition-p modes plist)
+           append mode-templates))
 
 (defun tempel--condition-p (modes plist)
   "Return non-nil if one of MODES matches and the PLIST condition is satisfied."
@@ -708,14 +730,18 @@ Capf, otherwise like an interactive completion command."
       (let ((completion-at-point-functions (list #'tempel-complete))
             completion-cycle-threshold)
         (tempel--save)
-        (when (and tempel-trigger-prefix (not (tempel--prefix-bounds)))
-          (insert tempel-trigger-prefix))
         (unless (completion-at-point)
           (user-error "tempel-complete: No matching templates")))
-    (let ((region (tempel--region)))
+    ;; If Tempel completion is invoked manually, use the marked region for
+    ;; template insertion. Insert a trigger prefix if missing. Furthermore
+    ;; accept empty input bounds.
+    (let* ((manually (memq this-command (list #'completion-at-point #'tempel-complete)))
+           (region (and manually (tempel--region))))
+      (when (and manually tempel-trigger-prefix (not (tempel--prefix-bounds)))
+        (insert tempel-trigger-prefix))
       (when-let ((templates (tempel--templates))
                  (bounds (or (and (not region) (tempel--prefix-bounds))
-                             (and (not tempel-trigger-prefix) (cons (point) (point))))))
+                             (and manually (cons (point) (point))))))
         (list (car bounds) (cdr bounds) templates
               :category 'tempel
               :exclusive 'no
@@ -726,11 +752,15 @@ Capf, otherwise like an interactive completion command."
               (apply-partially #'tempel--info-buffer templates
                                (lambda (elts)
                                  (insert (tempel--print-template elts))
+                                 (tempel--insert-doc elts)
                                  (current-buffer)))
               :company-location
               (apply-partially #'tempel--info-buffer templates
                                (lambda (elts)
-                                 (pp elts (current-buffer))
+                                 (pp (cl-loop for x in elts
+                                              until (keywordp x) collect x)
+                                     (current-buffer))
+                                 (tempel--insert-doc elts)
                                  (list (current-buffer))))
               :annotation-function
               (and tempel-complete-annotation
